// Add Quit Menu Entry to Popover
// Copyright (C) 2012 Joern Konopka
// Convenienve Functions stolen from windowoverlay-icons@sustmidown.centrum.cz

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

const Lang = imports.lang;
const St = imports.gi.St;
const Atk = imports.gi.Atk;
const Signals = imports.signals;

const AppDisplay = imports.ui.appDisplay;
const PopupMenu  = imports.ui.popupMenu;
const AppFavorites = imports.ui.appFavorites;

let quitfromDashInjections;

//Custom
const PopupButtonMenuItem = new Lang.Class({
	Name: 'PopupButtonMenuItem',
	Extends: PopupMenu.PopupBaseMenuItem,
	_init: function(text,iconName, params){
		this.parent(params);
		
		this.label = new St.Label({ text: text });
		this._button = new St.BoxLayout({ style_class: 'popup-menu-button-with-icon',
										  accessible_role: Atk.Role.PUSH_BUTTON,
										  hover: true,
										  reactive: true,
										  can_focus: true });
		
		
		this._icon = new St.Icon({ style_class: 'popup-menu-icon' });
		this._icon.icon_name = iconName;
		
		this._button.add_actor(this._icon);
				
        this.addActor(this.label);
        this.addActor(this._button, { align: St.Align.END });

		this._button.opacity = 0;
		this._button.active = false;	
		
		this._button.connect('button-press-event', Lang.bind(this, this._onButtonPress));
		this._button.connect('enter-event', Lang.bind(this, this._onButtonEnter));
		this._button.connect('leave-event', Lang.bind(this, this._onButtonLeave));
		this.actor.connect('enter-event', Lang.bind(this, this._onItemEnter));
		this.actor.connect('leave-event', Lang.bind(this, this._onItemLeave));
		
       	},
    _onButtonPress: function(actor, event){
		this._window.delete(global.get_current_time());
		this.destroy();
		//TODO: check if the first child in the menu is a separator, if true, delete it for a consistent look
		//TODO: check if two separators are siblings and remove one for consistent looks
		},
	_onButtonEnter: function(actor,event){
		this._button.active = true;
		},
	_onButtonLeave: function(actor,event){
		this._button.active = false;
		},
	_onItemEnter: function(actor,event){
		if(this._button.active == true){
			this._button.opacity = 150;
		}else{
			this._button.opacity = 70;
			}
		},
	_onItemLeave: function(actor,event){
		this._button.opacity = 0;
		}
		
});

// Convenience Functions
function injectToFunction(parent, name, func) {
    let origin = parent[name];
    parent[name] = function() {
        let ret;
        ret = origin.apply(this, arguments);
        if (ret === undefined)
                ret = func.apply(this, arguments);
        return ret;
    }
    return origin;
}

function removeInjection(object, injection, name) {
    if (injection[name] === undefined)
        delete object[name];
    else
        object[name] = injection[name];
}

function resetState() {
  quitfromDashInjections = { };
}

function enable() {
	resetState();
    
    quitfromDashInjections['_redisplay'] = undefined;
    quitfromDashInjections['_onActivate'] = undefined;
	
	quitfromDashInjections['_redisplay'] = injectToFunction(AppDisplay.AppIconMenu.prototype, '_redisplay', function() {
		this.removeAll();

		let windows = this._source.app.get_windows();

		// Display the app windows menu items and the separator between windows
        // of the current desktop and other windows.
        let activeWorkspace = global.screen.get_active_workspace();
        let separatorShown = windows.length > 0 && windows[0].get_workspace() != activeWorkspace;

        for (let i = 0; i < windows.length; i++) {
            if (!separatorShown && windows[i].get_workspace() != activeWorkspace) {
                this._appendSeparator();
                separatorShown = true;
            }
            let win = new PopupButtonMenuItem(windows[i].title, "edit-delete-symbolic", {});
            win._window = windows[i];
            this.addMenuItem(win);
            
        }

        if (!this._source.app.is_window_backed()) {
            if (windows.length > 0)
                this._appendSeparator();

            let isFavorite = AppFavorites.getAppFavorites().isFavorite(this._source.app.get_id());

            this._newWindowMenuItem = this._appendMenuItem(_("New Window"));
            this._appendSeparator();

            this._toggleFavoriteMenuItem = this._appendMenuItem(isFavorite ? _("Remove from Favorites") : _("Add to Favorites"));
        }
			
				let app = this._source.app;
				let count = app.get_n_windows();
				
				this._quitfromDashMenuItem = undefined;
				
				if(count == 1){
					this._appendSeparator();
					this._quitfromDashMenuItem = this._appendMenuItem(_("Quit")); 
				}else if(count > 1){
					this._appendSeparator();
					this._quitfromDashMenuItem = this._appendMenuItem(_("Quit " + count + " Windows")); 
				
					}
				
	});
	  
		  
	quitfromDashInjections['_onActivate'] = injectToFunction(AppDisplay.AppIconMenu.prototype, '_onActivate', function(actor, child) {
		if (child == this._quitfromDashMenuItem) {
			 let app = this._source.app;
		     let windows = app.get_windows();
		
			for (let i = 0; i < windows.length; i++) {
					windows[i].delete(global.get_current_time());
            }
		}
		
		if(child == this._testingMenuItem){
			
			}
		
	});
	
			
}
	

function disable() {
	 for (i in quitfromDashInjections) {
        removeInjection(AppDisplay.AppIconMenu.prototype, quitfromDashInjections, i);
		}
	resetState();
}

function init() {
    /* do nothing */
}
